@@ -695,6 +695,14 @@ class BaseDataStore:
         window.document.addEventListener(
             "visibilitychange", lambda: self.sync_soon(1.0), False
         )
+        window.addEventListener("beforeunload", self._beforeunload, False)
+
+    def _beforeunload(self, ev):
+        if self.state == "pending":
+            msg = "The most recent changes have not yet been stored."
+            ev.returnValue = msg
+            return msg
+        return None
 
     def reset(self):
         # The sub stores
